/*
 * This program converts the input signal to an m-frame 
 * transport smoothed signal for an arbitrarily selected
 * value of m. It produces a file with frame sizes, rates,
 * and duration for the m-frame transport signal.
 *
 * Juan Miguel del Rosario, (c) 1995.
 * All rights reserved.
 */

static char rcsid[] = "$Id: scanvbr.c,v 1.3 1995/06/24 18:24:55 mrosario Exp mrosario $";

/* $Log: scanvbr.c,v $
 * Revision 1.3  1995/06/24  18:24:55  mrosario
 * Removed header warning at end of program.  jmvr
 *
 * Revision 1.2  1995/06/24  18:00:57  mrosario
 * Modified to accept rate factor to 2nd decimal.   jmvr
 *
 * Revision 1.1  1995/06/24  15:51:07  mrosario
 * Initial revision
 * */


#include <stdio.h>
#include <string.h>
#include <math.h>

double statistics();

#ifdef NOTSELFRUN
scanvbr(filename, m, addrate) 
  char *filename;
  int m;
  int addrate;       /* see note on addrate below */
#else 
main(argc, argv)
  int argc;
  char *argv[];
#endif
{
   FILE *fd;
   FILE *fd1;
   FILE *fdo;
   int M,j,k;
   char fname[15];
   double mframe;
   double rate, time;
   int fsize, indx;
   double ratefactor;
   double stddev;
   
#ifdef NOTSELFRUN
   strcpy(fname, filename);
   M = m;
   ratefactor = addrate/100.0;
#else
   if (argc < 4) {
      printf("Usage: scanvbr [filename] [m] [rate factor]\n");
      printf("\n       The rate factor is an integer, usually [100, -100].\n");
      printf("       The actual transmission rate will be computed as: \n");
      printf("       TranRate = AveRate + (stddev * (rateFactor/100))\n");
      exit(1);
   } else {
      strcpy(fname, argv[1]);    
      M = (int) atoi(argv[2]);
      ratefactor = atoi(argv[3])/100.0;
   }
#endif

   fd = fopen(fname, "r");
   if (fd == 0) {
      printf("Some error on open\n");
   }
   fd1 = fopen(fname, "r");
   if (fd1 == 0) {
      printf("Some error on open\n");
   }
   fdo = fopen("mfvbr.dat", "wt");
   if (fdo == 0) {
      printf("some error opening\n");
   }

  /*
   * compute stddev of the stream and multiply by the rate factor 
   * to get the desired rate adjustment to apply to the average rate.
   * 
   * i.e.,  actual transmission rate =  stddev * ratefactor;
   */
   stddev = statistics(fd);

   printf("Rate Factor               : %e\n", ratefactor);
   printf("Rate Modification Factor  : %e\n", stddev*ratefactor);
   ratefactor = ratefactor * stddev;

#ifdef DEBUG
   fprintf(fdo, "+----------------------------\n");
   fprintf(fdo, "| M = %d\n");
   fprintf(fdo, "| indx  frame  rate  duration\n");
   fprintf(fdo, "+----------------------------\n");
#endif 

   fseek(fd,0,0);
   while (!feof(fd)) {
      mframe = 0;
      j = 1;
      k = 0;
      while (j <= M) {
        fscanf(fd,"%d %d", &indx, &fsize);
        if (feof(fd)) {
           break; 
        }
        mframe = mframe + (fsize*1.0);
        j++; 
        k++;
      }
#if 0
      printf("Average Bit Rate (bps)    : %e\n", (mframe/k)*30);
#endif
      rate = (mframe/k)*30 + ratefactor;
      for (j=1; j <= k; j++) {
         fscanf(fd1,"%d %d", &indx, &fsize);
         time = fsize/rate;
         fprintf(fdo,"%d %d %e %e\n", indx, fsize, rate, time);
/* fprintf(fdo,"%4d  %6d   %9.2f   %2.4f\n", indx, fsize, rate, time); */
      }
   }
   fclose(fd);
   fclose(fd1);
   fclose(fdo);
}

/*------------------------------------------------------*/
double statistics(fd) 
   FILE *fd;
{
   int fno, fsize;
   double maxframesize;
   double framesize, framecount, totframesize;
   double framesize_sqr;
   double meanframesize, variance, stddev;

   fseek(fd,0,0);
   totframesize = maxframesize = 0.0;
   framesize_sqr = framecount = 0.0;
   while (!feof(fd)) {
      fscanf(fd, "%d %d\n", &fno, &fsize);
      framecount += 1.0;
      totframesize += fsize;
      framesize = fsize;
      framesize_sqr += pow(framesize,2.0);
      if (framesize > maxframesize) maxframesize = framesize;
   }
   fseek(fd,0,0);

   meanframesize = totframesize/framecount;
   variance = (framesize_sqr/framecount) - pow(meanframesize,2.0);
   stddev = sqrt(variance);

   printf("FRAME SIZE STATISTICS:\n"); 
   printf("Max. Frame Size           : %e\n", maxframesize);
   printf("Ave. Frame Size           : %e\n", meanframesize);
   printf("Frame Size Variance       : %e\n", variance);
   printf("Frame Size Std. Deviation : %e\n", stddev);

   return(stddev);
}








